package com.strangemachinery.billboard;

import java.io.IOException;
import java.util.ArrayList;


import com.strangemachinery.billboard.R;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

public class BillboardList extends ListActivity
{
	private static final int DIALOG_EDIT = 1;
	private static final int DIALOG_DELETE = 2;
	private static final int DIALOG_PICKCOLOR = 3;
	private static final int DIALOG_HELP = 4;
	private static final int MENU_ADD = 1;
	
	private int[] idList;
	private String[] billboardList;
	private String[] foregroundList;
	private String[] backgroundList;
	private int selectedIndex;
	
	private DBAdapter database;
	
	public void onCreate(Bundle savedInstanceState)
	{
		Log.d("Billboard", "Starting Billboard...");
		
		// On first run
		if (getSharedPreferences("PREFERENCE", MODE_PRIVATE).getBoolean("firstrun", true))
		{
			//Display help dialog
			showDialog(DIALOG_HELP);
			
			// Save the state
			getSharedPreferences("PREFERENCE", MODE_PRIVATE)
			.edit()
			.putBoolean("firstrun", false)
			.commit();
		}
		
		super.onCreate(savedInstanceState);
		
		connectToDB();
		
		refreshBillboardList();
		
		selectedIndex = -1;
		
		// Create the view, and allow the user to type to sort the list
		//  I have no idea why anyone would want to do that, but they totally can now.
		ListView lv = getListView();
		lv.setTextFilterEnabled(true);
		
		// Respond to the user's selection
		lv.setOnItemClickListener(
			new OnItemClickListener()
			{
				public void onItemClick(AdapterView<?> parent, View view, int position, long id)
				{
					Intent billboardIntent = new Intent(view.getContext(), BillboardViewActivity.class);
					
					// Add in the extras from the database
					billboardIntent.putExtra("text", billboardList[position]);
					if(foregroundList[position] != null) billboardIntent.putExtra("foreground", Integer.parseInt(foregroundList[position]));
					if(backgroundList[position] != null) billboardIntent.putExtra("background", Integer.parseInt(backgroundList[position]));
					
					if(billboardIntent.getExtras() != null)
						startActivity(billboardIntent);
				}
			}
		);
		
		registerForContextMenu(lv);
	}
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View view,ContextMenuInfo menuInfo)
	{
		super.onCreateContextMenu(menu, view, menuInfo);
		
		selectedIndex = ((AdapterContextMenuInfo)menuInfo).position;
		
		//Toast.makeText(this, "SelectedIndex:" + selectedIndex, Toast.LENGTH_LONG).show();
		
		// Set the title to include the billboard's text
		menu.setHeaderTitle("\"" + billboardList[selectedIndex] + "\" billboard");
		
		
		
		// Build the menu
		menu.add(0, 0, 0, "Edit");
		menu.add(0, 1, 0, "Delete");
	}
	
	@Override
	public boolean onContextItemSelected(MenuItem item)
	{
		if(item.getItemId() == 0)
		{
			// 'Edit billboard' selected
			showDialog(DIALOG_EDIT);
		}
		else if(item.getItemId() == 1)
		{
			// 'Delete billboard' selected
			showDialog(DIALOG_DELETE);
		}
		else 
		{
			return false;
		}
		
		//selectedIndex = -1;
		
		return true;
	}
	
	@Override protected Dialog onCreateDialog(int id) {
		//Toast.makeText(this, "SelectedIndex:" + selectedIndex, Toast.LENGTH_LONG).show();
		switch (id)
		{
			case DIALOG_DELETE:
				return new AlertDialog.Builder(this)
					//.setIconAttribute(android.R.attr.alertDialogIcon)
					.setTitle(R.string.dialog_delete_title)
					.setMessage(R.string.dialog_delete_body)
					.setPositiveButton(R.string.dialog_delete_yes, new DialogInterface.OnClickListener(){
						public void onClick(DialogInterface dialog, int whichButton)
						{
							// User clicked delete
							deleteSelectedBillboard();
							refreshBillboardList();
							selectedIndex = -1;
							removeDialog(DIALOG_DELETE);
						}
					})
					.setNegativeButton(R.string.dialog_delete_no, new DialogInterface.OnClickListener(){
						public void onClick(DialogInterface dialog, int whichButton)
						{
							// User clicked cancel, say fuggit and bail
							selectedIndex = -1;
							removeDialog(DIALOG_DELETE);
						}
					})
					.create();
			case DIALOG_EDIT:
				final View editDialogView = LayoutInflater.from(this).inflate(R.layout.edit_dialog, null);
				
				if(selectedIndex != -1)
					((EditText)editDialogView.findViewById(R.id.tbx_text)).setText(billboardList[selectedIndex]);
				else
					((EditText)editDialogView.findViewById(R.id.tbx_text)).setText("");
				
				return new AlertDialog.Builder(BillboardList.this)
				.setTitle(R.string.dialog_edit_title)
				.setView(editDialogView)
				.setPositiveButton(R.string.dialog_edit_yes, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton)
					{
						// User clicked save
						String text = ((EditText) editDialogView.findViewById(R.id.tbx_text)).getText().toString();
						//String fgColor = ((EditText) editDialogView.findViewById(R.id.tbx_fgcolor)).getText().toString();
						//String bgColor = ((EditText) editDialogView.findViewById(R.id.tbx_bgcolor)).getText().toString();
						
						//editBillboard(text, fgColor, bgColor);
						editBillboard(text, null, null);
						
						selectedIndex = -1;
						removeDialog(DIALOG_EDIT);
					}
				})
				.setNegativeButton(R.string.dialog_edit_no, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton)
					{
						// User clicked cancel, say fuggit and bail
						//Toast.makeText(BillboardList.this, "Sayin' fuggit, bailing", Toast.LENGTH_SHORT).show();
						selectedIndex = -1;
						removeDialog(DIALOG_EDIT);
					}
				})
				.create();
			case DIALOG_HELP:
				return new AlertDialog.Builder(this)
				.setTitle(R.string.dialog_help_title)
				.setMessage(R.string.dialog_help_body)
				.setNeutralButton(R.string.dialog_help_ok, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int which) {
						// Don't do anything
					}
				})
				.create();
		}
		
		selectedIndex = -1;
		
		//removeDialog(id);
		
		return null;
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		menu.add(0, MENU_ADD, 0, "Add new billboard");
		return super.onCreateOptionsMenu(menu);
	}
	
	@Override public boolean onOptionsItemSelected (MenuItem item)
	{
		switch(item.getItemId())
		{
			case MENU_ADD:
				showDialog(DIALOG_EDIT);
		}
		
		selectedIndex = -1;
		
		return true;
	}
	
	public boolean pickColor(View view)
	{
		showDialog(DIALOG_PICKCOLOR);
		
		return true;
	}
	
	public boolean connectToDB()
	{
		// Connect to the database
		database = DBAdapter.getDBAdapterInstance(this);
		try
		{
			database.createDataBase();
		}
		catch (IOException e)
		{
			e.printStackTrace();
			return false;
		}
		database.openDataBase();
		
		return true;
	}
	
	public boolean refreshBillboardList()
	{
		// Get the contents of the database
		//final ArrayList<ArrayList<String>> results = database.listBillboards();
		String[] columns = {"billboard_id", "billboard_text", "billboard_foreground", "billboard_background"};
		ArrayList<ArrayList<String>> results = database.selectRecordsFromDBList("billboards", columns, null, null, null, null, null);
		
		// Fill an array of strings for the listview
		idList = new int[results.size()];
		billboardList = new String[results.size()];
		foregroundList = new String[results.size()];
		backgroundList = new String[results.size()];
		for(int count = 0; count < results.size(); count++)
		{
			idList[count] = Integer.parseInt(results.get(count).get(0));
			billboardList[count] = results.get(count).get(1);
			foregroundList[count] = results.get(count).get(2);
			backgroundList[count] = results.get(count).get(3);
		}
		setListAdapter(new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, billboardList));
		
		return true;
	}
	
	public boolean createBillboard(String text, String fgColor, String bgColor)
	{
		return true;
	}
	
	public boolean editBillboard(String text, String fgColor, String bgColor)
	{
		if(selectedIndex == -1)
		{
			// Create a new billboard
			
			ContentValues args = new ContentValues();
			args.put("billboard_text", text);
			//args.put("billboard_foreground", 0);
			//args.put("billboard_background", 0);
			
			database.insertRecordInDB("billboards", args);
		}
		else
		{
			// Edit an existing billboard
			
			Log.v("billboard", "selectedIndex=" + selectedIndex + ", id=" + idList[selectedIndex]);
			
			String filter = "billboard_id=" + idList[selectedIndex];
			ContentValues args = new ContentValues();
			args.put("billboard_text", text);
			//args.put("billboard_foreground", 0);
			//args.put("billboard_background", 0);
			
			database.updateRecordsInDB("billboards", args, filter, null);
			
			selectedIndex = -1;
		}
		
		refreshBillboardList();
		
		return true;
	}
	
	public boolean deleteSelectedBillboard()
	{
		Toast.makeText(this, "CONSIDER THAT BILLBOARD (" + selectedIndex + ") OOUUUTTTAAAA HEEEERRREE", Toast.LENGTH_SHORT).show();
		Log.e("billboard", "CONSIDER THAT BILLBOARD (" + selectedIndex + ") OOUUUTTTAAAA HEEEERRREE");
		database.deleteRecordInDB("billboards", "billboard_id=" + idList[selectedIndex], null);
		
		refreshBillboardList();
		
		selectedIndex = -1;
		
		return true;
	}
}
