package com.strangemachinery.billboard;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Typeface;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;

class BillboardView extends View
{
	private String text;
	private Paint background;
	private Paint foreground;
	
	public BillboardView(Context context)
	{
		super(context);
		
		this.text = null;
		this.background = null;
		this.foreground = null;
		
		Log.d("Billboard", "Created a null BillboardView.");
	}
	
	public BillboardView(Context context, String text)
	{
		super(context);
		
		this.text = text;
		this.background = null;
		this.foreground = null;
		
		Log.d("Billboard", "Created a null BillboardView with text \"" + text + "\".");
	}
	
	public BillboardView(Context context, String text, int foregroundColor, int backgroundColor)
	{
		super(context);
		
		this.text = text;
		this.foreground = new Paint();
		this.foreground.setColor(foregroundColor);
		this.background = new Paint();
		this.background.setColor(backgroundColor);
		
		Log.d("Billboard", "Created a BillboardView with text \"" + text + "\", background " + backgroundColor + ", and foreground " + foregroundColor);
	}
	
	@Override
	protected void onDraw(Canvas canvas)
	{
		// Handle null text
		if(text == null)
		{
			Log.d("Billboard", "Text is null.");
			text = "null";
		}
		
		// Handle null background color
		if(background == null)
		{
			Log.d("Billboard", "Background is null.");
			background = new Paint();
			background.setColor(Color.RED);
			
		}
		
		// Handle null foreground color
		if(foreground == null)
		{
			Log.d("Billboard", "Foreground is null.");
			foreground = new Paint();
			foreground.setColor(Color.WHITE);
		}
		
		// Draw the background
		canvas.drawPaint(background);
		
		// Set some foreground properties
		foreground.setAntiAlias(true);
		foreground.setSubpixelText(true);
		foreground.setTypeface(Typeface.DEFAULT_BOLD);

		// Declare variables that are used in computing the maximum size for the text
		int screenWidth = ((WindowManager) this.getContext().getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay().getWidth();
		int screenHeight = ((WindowManager) this.getContext().getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay().getHeight();
		int textSize = 300;
		Rect textBounds = new Rect();
		
		// Figure out the largest size text possible, by brute-force trial-and-error
		while(screenWidth < textBounds.right || textBounds.right == 0)
		{
			textSize--;
			foreground.setTextSize(textSize);
			//textWidth = foreground.measureText(text);
			foreground.getTextBounds(text, 0, text.length(), textBounds);
		}
		
		// Draw the text in the middle of the screen
		canvas.drawText(text, (screenWidth / 2) - (textBounds.right / 2), (screenHeight / 2) - (textBounds.top / 2), foreground);
	}
}
