/******************************************************************************
Code originally from http://mfarhan133.wordpress.com/2010/10/24/database-crud-tutorial-for-android/,
and modified by me (especially because portions were unfinished or innefficient)
******************************************************************************/

package com.strangemachinery.billboard;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;

public class DBAdapter extends SQLiteOpenHelper
{

	private static String DB_PATH = "/assets/";
	private static final String DB_NAME = "billboards.sqlite";
	private SQLiteDatabase dataBase;
	private final Context myContext;
	
	private static DBAdapter mDBConnection;
	
	// Constructor
	private DBAdapter(Context context)
	{
		super(context, DB_NAME, null, 1);
		
		this.myContext = context;
		
		// The Android's default system path of your application database is "/data/data/mypackagename/databases/"
		DB_PATH = "/data/data/" + context.getApplicationContext().getPackageName() + "/databases/";
	}
	
	// Get's an instance of the adapter
	public static synchronized DBAdapter getDBAdapterInstance(Context context)
	{
		if (mDBConnection == null)
		{
			mDBConnection = new DBAdapter(context);
		}
		
		return mDBConnection;
	}
	
	// Creates an empty database on the system and rewrites it with your own database.
	public void createDataBase() throws IOException
	{
		if(!checkDataBase())
		{
			// Create a new, empty database for this app
			this.getReadableDatabase();
			
			try
			{
				// Attempt to overwrite the new database with our initial database from the /assets/ folder
				copyDataBase();
			}
			catch (IOException e)
			{
				throw new Error("Error copying database");
			}
		}
	}
	
	// Check if the database already exists to avoid re-copying the file each time you open the application.
	//  Returns true if it exists, false otherwise.
	private boolean checkDataBase()
	{
		SQLiteDatabase checkDB = null;
		
		try
		{
			// Try to connect to the database
			
			String myPath = DB_PATH + DB_NAME;
			checkDB = SQLiteDatabase.openDatabase(myPath, null, SQLiteDatabase.OPEN_READONLY);
			
			checkDB.close();
			
			return true;
		}
		catch (SQLiteException e)
		{
			// Database does't exist yet.
			return false;
		}
	}
	
	// Copies the initial-state database from the /assets/ folder over the database in the system folder
	private void copyDataBase() throws IOException
	{
		// Open your local db as the input stream
		InputStream myInput = myContext.getAssets().open(DB_NAME);
		
		// Path to the just created empty db
		String outFileName = DB_PATH + DB_NAME;
		
		// Open the empty db as the output stream
		OutputStream myOutput = new FileOutputStream(outFileName);
		
		// transfer bytes from the inputfile to the outputfile
		byte[] buffer = new byte[1024];
		int length;
		while ((length = myInput.read(buffer)) > 0)
		{
			myOutput.write(buffer, 0, length);
		}
		
		// Close the streams
		myOutput.flush();
		myOutput.close();
		myInput.close();
	}
	
	// Opens the database from the default location
	public void openDataBase() throws SQLException
	{
		dataBase = SQLiteDatabase.openDatabase(
			DB_PATH + DB_NAME,
			null, SQLiteDatabase.OPEN_READWRITE
		);
	}

	// Closes the database.
	@Override
	public synchronized void close()
	{
		if (dataBase != null)
		{
			dataBase.close();
		}
		
		super.close();
	}

	// Called on creation of the database, useful for creating tables at run time
	@Override
	public void onCreate(SQLiteDatabase db) {}
	
	// Called when the database version is updated, useful for modifying tables
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {}

	/************************* CRUD Functions ********************************/

	// This function is used to select records from the database. Returns a Cursor object positioned before the first entry
	public Cursor selectRecordsFromDB(String tableName, String[] tableColumns, String whereClase, String whereArgs[], String groupBy, String having, String orderBy)
	{
		return dataBase.query(tableName, tableColumns, whereClase, whereArgs, groupBy, having, orderBy);
	}
	
	// This function is used to select records from the database, and return them in the form of an ArrayList<ArrayList<String>>
	public ArrayList<ArrayList<String>> selectRecordsFromDBList(String tableName, String[] tableColumns, String whereClase, String whereArgs[], String groupBy, String having, String orderBy)
	{
		ArrayList<ArrayList<String>> retList = new ArrayList<ArrayList<String>>();
		ArrayList<String> list = new ArrayList<String>();
		Cursor cursor = dataBase.query(tableName, tableColumns, whereClase, whereArgs, groupBy, having, orderBy);
		
		if (cursor.moveToFirst())
		{
			do{
				list = new ArrayList<String>();
				for(int i = 0; i < cursor.getColumnCount(); i++)
				{
					list.add(cursor.getString(i));
				}
				retList.add(list);
			} while(cursor.moveToNext());
		}
		
		if (cursor != null && !cursor.isClosed())
		{
			cursor.close();
		}
		
		return retList;
	}
	
	// This function updates a record in the database, then returns the number of rows affected.
	public int updateRecordsInDB(String tableName, ContentValues initialValues, String whereClause, String whereArgs[])
	{
		return dataBase.update(tableName, initialValues, whereClause, whereArgs);
	}
	
	// This function deletes a record from the database, then returns the number of rows affected.
	public int deleteRecordInDB(String tableName, String whereClause, String[] whereArgs)
	{
		return dataBase.delete(tableName, whereClause, whereArgs);
	}
	
	public long insertRecordInDB(String tableName, ContentValues newValues)
	{
		return dataBase.insert(tableName, null, newValues);
	}
	
	/************************Select Raw Query Functions **********************/
	
	// Run a raw query on the database. Returns a Cursor object pointing to the before the first result of the query.
	public Cursor selectRecordsFromDB(String query, String[] selectionArgs)
	{
		return dataBase.rawQuery(query, selectionArgs);
	}
	
	// Run a raw query on the database. Returns an ArrayList<ArrayList<String>> of the results.
	public ArrayList<ArrayList<String>> selectRecordsFromDBList(String query, String[] selectionArgs)
	{
		ArrayList<ArrayList<String>> retList = new ArrayList<ArrayList<String>>();
		ArrayList<String> list = new ArrayList<String>();
		Cursor cursor = dataBase.rawQuery(query, selectionArgs);
		
		if (cursor.moveToFirst())
		{
			do{
				list = new ArrayList<String>();
				for(int i = 0; i < cursor.getColumnCount(); i++)
				{
					list.add(cursor.getString(i));
				}
				retList.add(list);
			} while(cursor.moveToNext());
		}
		
		if (cursor != null && !cursor.isClosed())
		{
			cursor.close();
		}
		
		return retList;
	}
}
